@@ -38,6 +38,7 @@ from flax.linen import module as linen_module
 from flax.linen.module import Module
 from flax.linen.module import Variable
 from flax.linen.module import wrap_method_once
+from flax.linen.module import _get_unbound_fn
 import jax
 
 traceback_util.register_exclusion(__file__)
@@ -290,6 +291,7 @@ def module_class_lift_transform(
     # we need to create a scope-function from our class for the given method
     @functools.wraps(fn)
     def wrapped_fn(self, *args, **kwargs):
+      state = self._state.export()
       # make a scope-function to transform
       def core_fn(scopes, *args, **kwargs):
         # make a clone of self using its arguments
@@ -301,7 +303,7 @@ def module_class_lift_transform(
         # we reference module_class, not self.__class__ to avoid infinite loop
         cloned = module_class(parent=None, **attrs)
         cloned, args, kwargs = set_module_scopes(cloned, args, kwargs, scopes)
-        object.__setattr__(cloned, '_state', self._state.export())  # pylint: disable=protected-access
+        object.__setattr__(cloned, '_state', state.export())  # pylint: disable=protected-access
         res = fn(cloned, *args, **kwargs)
         self._state.reimport(cloned._state)  # pylint: disable=protected-access
         _test_transformed_return_values(res, fn_name)
@@ -343,12 +345,13 @@ def decorator_lift_transform(transform, class_fn, *trafo_args,
   prewrapped_fns = [wrap_method_once(class_fn) for class_fn in class_fns]
   @functools.wraps(prewrapped_fns[0])
   def wrapped_fn(self, *args, **kwargs):
+    state = self._state.export()
     # make a scope-function to transform
     def core_fn(prewrapped_fn, class_fn, scopes, *args, **kwargs):
       if not multi_scope:
         scopes = [scopes]
       cloned, args, kwargs = set_module_scopes(self, args, kwargs, scopes)
-      object.__setattr__(cloned, '_state', self._state.export())  # pylint: disable=protected-access
+      object.__setattr__(cloned, '_state', state.export())  # pylint: disable=protected-access
       res = prewrapped_fn(cloned, *args, **kwargs)
       self._state.reimport(cloned._state)  # pylint: disable=protected-access
       _test_transformed_return_values(res, getattr(class_fn, '__name__', None))
@@ -417,6 +420,8 @@ def lift_direct_transform(transform: Callable[..., Any],
           ' That is function that takes a Module instance as its first arg.')
     elif not callable(target):
       raise ValueError('transform target must be callable')
+  # normalize self.foo bound methods to class.foo unbound methods.
+  targets = tuple(_get_unbound_fn(target) for target in targets)
   aug_transform = lambda *fns: functools.partial(transform, *fns)
   return decorator_lift_transform(
       aug_transform, targets, multi_scope=multi_scope)(mdl, *args, **kwargs)
@@ -669,6 +674,7 @@ def scan(target: Target,
          in_axes=0, out_axes=0,
          length: Optional[int] = None,
          reverse: bool = False,
+         unroll: int = 1,
          data_transform: Optional[Callable[..., Any]] = None,
          methods=None) -> Target:
   """A lifted version of ``jax.lax.scan``.
@@ -761,6 +767,8 @@ def scan(target: Target,
     length: Specifies the number of loop iterations. This only needs to be
       specified if it cannot be derivied from the scan arguments.
     reverse: If true, scan from end to start in reverse order.
+    unroll: how many scan iterations to unroll within a single
+      iteration of a loop (default: 1).
     data_transform: optional function to transform raw functional-core variable
       and rng groups inside lifted scan body_fn, intended for inline SPMD
       annotations.
@@ -780,6 +788,7 @@ def scan(target: Target,
       in_axes=in_axes, out_axes=out_axes,
       length=length,
       reverse=reverse,
+      unroll=1,
       data_transform=data_transform,
       methods=methods)
 
@@ -1057,6 +1066,63 @@ def while_loop(
       split_rngs)
 
 
+def _cond_wrapper(t_fn, f_fn, scope, pred, *ops, variables, rngs):
+  return lift.cond(pred, t_fn, f_fn, scope, *ops, variables=variables, rngs=rngs)
+
+
+def cond(
+    pred: Any,
+    true_fun: Callable[..., C], false_fun: Callable[..., C],
+    mdl: Module, *operands,
+    variables: lift.CollectionFilter = True,
+    rngs: lift.PRNGSequenceFilter = True) -> C:
+  """Lifted version of ``jax.lax.cond``.
+
+  The returned values from ``true_fun`` and ``false_fun``
+  must have the same Pytree structure, shapes, and dtypes.
+  The variables created or updated inside the
+  branches must also have the same structure.
+  Note that this constraint is violated when
+  creating variables or submodules in only one branch.
+  Because initializing variables in just one branch
+  causes the parameter structure to be different.
+
+  Example::
+
+    class CondExample(nn.Module):
+      @nn.compact
+      def __call__(self, x, pred):
+        self.variable('state', 'true_count', lambda: 0)
+        self.variable('state', 'false_count', lambda: 0)
+        def true_fn(mdl, x):
+          mdl.variable('state', 'true_count').value += 1
+          return nn.Dense(2, name='dense')(x)
+        def false_fn(mdl, x):
+          mdl.variable('state', 'false_count').value += 1
+          return -nn.Dense(2, name='dense')(x)
+        return nn.cond(pred, true_fn, false_fn, self, x)
+
+
+  Args:
+    pred: determines if true_fun or false_fun is evaluated.
+    true_fun: The function evalauted when ``pred`` is `True`.
+      The signature is (module, *operands) -> T.
+    false_fun: The function evalauted when ``pred`` is `False`.
+      The signature is (module, *operands) -> T.
+    mdl: A Module target to pass.
+    *operands: The arguments passed to ``true_fun`` and ``false_fun``
+    variables: The variable collections passed to the conditional
+      branches (default: all)
+    rngs: The PRNG sequences passed to the conditionals (default: all)
+  Returns:
+    The result of the evaluated branch (``true_fun`` or ``false_fun``).
+  """
+  return lift_direct_transform(
+      _cond_wrapper, (true_fun, false_fun), mdl,
+      pred, *operands,
+      variables=variables, rngs=rngs)
+
+
 # a version of lift.custom_vjp with a single scope function
 # this avoids having to lift multiple functions in
 # lift_transform.
@@ -1156,28 +1222,43 @@ def named_call(class_fn, force=True):
   Returns:
     A wrapped version of ``class_fn`` that is labeled.
   """
-  # Due to the ordering of method decorators, we must wrap the class_fn
-  # with the module state management wrapper first to maintain Module state
-  # correctly.
-  prewrapped_fn = wrap_method_once(class_fn)
-  @functools.wraps(prewrapped_fn)
-  def wrapped_fn(self, *args, **kwargs):
-    if (not force and not linen_module._use_named_call) or self._state.in_setup:  # pylint: disable=protected-access
-      return prewrapped_fn(self, *args, **kwargs)
-    fn_name = class_fn.__name__
-    method_suffix = f'.{fn_name}' if fn_name != '__call__' else ''
-    module_name = self.name or self.__class__.__name__
-    full_name = f'{module_name}{method_suffix}'
-    # make a scope-function to transform
-    def core_fn(scopes, *args, **kwargs):
-      cloned, args, kwargs = set_module_scopes(self, args, kwargs, scopes)
-      object.__setattr__(cloned, '_state', self._state.export())  # pylint: disable=protected-access
-      res = prewrapped_fn(cloned, *args, **kwargs)
-      self._state.reimport(cloned._state)  # pylint: disable=protected-access
-      _test_transformed_return_values(res, fn_name)
-      return res
-    # here we apply the given lifting transform to the scope-ingesting fn
-    trafo_fn = lift.named_call(core_fn, full_name)
-    module_scopes, args, kwargs = get_module_scopes(self, args, kwargs)
-    return trafo_fn(module_scopes, *args, **kwargs)
+  if (hasattr(jax.config, 'jax_experimental_name_stack') and
+      jax.config.jax_experimental_name_stack):
+    # Use JAX's improved dynamic name-stack named_call.
+    # No transform boundary needed!
+    @functools.wraps(class_fn)
+    def wrapped_fn(self, *args, **kwargs):
+      fn_name = class_fn.__name__
+      method_suffix = f'.{fn_name}' if fn_name != '__call__' else ''
+      module_name = self.name or self.__class__.__name__
+      full_name = f'{module_name}{method_suffix}'
+      return jax.named_call(class_fn, name=full_name)(self, *args, **kwargs)
+  else:
+    # Use JAX's old purely-functional call-based named_call.
+    # Due to the ordering of method decorators, we must wrap the class_fn
+    # with the module state management wrapper first to maintain Module state
+    # correctly.
+    prewrapped_fn = wrap_method_once(class_fn)
+    @functools.wraps(prewrapped_fn)
+    def wrapped_fn(self, *args, **kwargs):
+      if ((not force and not linen_module._use_named_call)  # pylint: disable=protected-access
+          or self._state.in_setup):  # pylint: disable=protected-access
+        return prewrapped_fn(self, *args, **kwargs)
+      fn_name = class_fn.__name__
+      method_suffix = f'.{fn_name}' if fn_name != '__call__' else ''
+      module_name = self.name or self.__class__.__name__
+      full_name = f'{module_name}{method_suffix}'
+      # make a scope-function to transform
+      def core_fn(scopes, *args, **kwargs):
+        cloned, args, kwargs = set_module_scopes(self, args, kwargs, scopes)
+        object.__setattr__(cloned, '_state', self._state.export())  # pylint: disable=protected-access
+        res = prewrapped_fn(cloned, *args, **kwargs)
+        self._state.reimport(cloned._state)  # pylint: disable=protected-access
+        _test_transformed_return_values(res, fn_name)
+        return res
+      # here we apply the given lifting transform to the scope-ingesting fn
+      trafo_fn = lift.named_call(core_fn, full_name)
+      module_scopes, args, kwargs = get_module_scopes(self, args, kwargs)
+      return trafo_fn(module_scopes, *args, **kwargs)
+
   return wrapped_fn
